@@ -1,4 +1,4 @@
-/* $Id: child-fetch.c,v 1.69 2008/09/29 16:58:36 nicm Exp $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -45,19 +45,14 @@ void	fetch_free1(struct mail_ctx *);
 int	fetch_enqueue(struct account *, struct io *, struct mail *);
 int	fetch_dequeue(struct account *, struct mail_ctx *);
 
-#ifdef DEBUG
-double			 fetch_time_polling = 0.0;
-double			 fetch_time_blocked = 0.0;
-#endif
+struct mail_queue	 fetch_matchq;
+struct mail_queue	 fetch_deliverq;
 
-struct mail_queue 	 fetch_matchq;
-struct mail_queue 	 fetch_deliverq;
+u_int			 fetch_dropped;
+u_int			 fetch_kept;
 
-u_int		  	 fetch_dropped;
-u_int		  	 fetch_kept;
-
-u_int			 fetch_queued;   /* number of mails queued */
-u_int			 fetch_blocked;  /* blocked for parent */
+u_int			 fetch_queued;	 /* number of mails queued */
+u_int			 fetch_blocked;	 /* blocked for parent */
 
 int
 open_cache(struct account *a, struct cache *cache)
@@ -94,20 +89,15 @@ int
 child_fetch(struct child *child, struct io *pio)
 {
 	struct child_fetch_data	*data = child->data;
-	enum fdmop 		 op = data->op;
-	struct account 		*a = data->account;
-	struct msg	 	 msg;
+	enum fdmop		 op = data->op;
+	struct account		*a = data->account;
+	struct msg		 msg;
 	int			 error, flags;
 	double			 tim;
 
-#ifdef DEBUG
-	xmalloc_clear();
-	COUNTFDS(a->name);
-#endif
-
 	log_debug2("%s: fetch started, pid %ld", a->name, (long) getpid());
 
-#ifndef NO_SETPROCTITLE
+#ifdef HAVE_SETPROCTITLE
 	setproctitle("child: %s", a->name);
 #endif
 
@@ -127,16 +117,11 @@ child_fetch(struct child *child, struct io *pio)
 	log_debug3("%s: sending exit message to parent", a->name);
 	if (privsep_send(pio, &msg, NULL) != 0)
 		fatalx("privsep_send error");
-	log_debug3("%s: waiting for exit message from parent", a->name);
-	if (privsep_recv(pio, &msg, NULL) != 0)
-		fatalx("privsep_recv error");
-	if (msg.type != MSG_EXIT)
-		fatalx("unexpected message");
-
-#ifdef DEBUG
-	COUNTFDS(a->name);
-	xmalloc_report(getpid(), a->name);
-#endif
+	do {
+		log_debug3("%s: waiting for exit message from parent", a->name);
+		if (privsep_recv(pio, &msg, NULL) != 0)
+			fatalx("privsep_recv error");
+	} while (msg.type != MSG_EXIT);
 
 	return (error);
 }
@@ -169,12 +154,6 @@ fetch_poll(struct account *a, struct iolist *iol, struct io *pio, int timeout)
 	}
 	tim = get_time() - tim;
 
-#ifdef DEBUG
-	fetch_time_polling += tim;
-	if (fetch_blocked == fetch_queued && fetch_queued != 0)
-		fetch_time_blocked += tim;
-#endif
-
 	return (0);
 }
 
@@ -195,16 +174,24 @@ fetch_match(struct account *a, struct msg *msg, struct msgbuf *msgbuf)
 		    "trying (match) message %u", a->name, this->mail->idx);
 		switch (mail_match(this, msg, msgbuf)) {
 		case MAIL_ERROR:
+			log_debug3("%s: match"
+			    " message %u, error", a->name, this->mail->idx);
 			return (-1);
 		case MAIL_DELIVER:
+			log_debug3("%s: match"
+			    " message %u, deliver", a->name, this->mail->idx);
 			TAILQ_REMOVE(&fetch_matchq, this, entry);
 			TAILQ_INSERT_TAIL(&fetch_deliverq, this, entry);
 			break;
 		case MAIL_DONE:
+			log_debug3("%s: match"
+			    " message %u, done", a->name, this->mail->idx);
 			if (fetch_dequeue(a, this) != 0)
 				return (-1);
 			break;
 		case MAIL_BLOCKED:
+			log_debug3("%s: match"
+			    " message %u, blocked", a->name, this->mail->idx);
 			fetch_blocked++;
 			break;
 		}
@@ -230,12 +217,32 @@ fetch_deliver(struct account *a, struct msg *msg, struct msgbuf *msgbuf)
 		    " trying (deliver) message %u", a->name, this->mail->idx);
 		switch (mail_deliver(this, msg, msgbuf)) {
 		case MAIL_ERROR:
+			log_debug3("%s: deliver"
+			    " message %u, error", a->name, this->mail->idx);
+
+			if (conf.ignore_errors) {
+				log_warnx("%s: fetching error. ignored",
+				    a->name);
+
+				TAILQ_REMOVE(&fetch_deliverq, this, entry);
+				TAILQ_INSERT_TAIL(&fetch_matchq, this, entry);
+
+				this->mail->decision = DECISION_KEEP;
+				if (fetch_dequeue(a, this) != 0)
+					return (-1);
+				break;
+			}
+
 			return (-1);
 		case MAIL_MATCH:
+			log_debug3("%s: deliver"
+			    " message %u, match", a->name, this->mail->idx);
 			TAILQ_REMOVE(&fetch_deliverq, this, entry);
 			TAILQ_INSERT_TAIL(&fetch_matchq, this, entry);
 			break;
 		case MAIL_BLOCKED:
+			log_debug3("%s: deliver"
+			    " message %u, blocked", a->name, this->mail->idx);
 			fetch_blocked++;
 			break;
 		}
@@ -318,11 +325,6 @@ fetch_status(struct account *a, double tim)
 		log_info("%s: 0 messages processed in %.3f seconds",
 		    a->name, tim);
 	}
-
-#ifdef DEBUG
-	log_debug("%s: polled for %.3f seconds (%.3f blocked)", a->name,
-	    fetch_time_polling, fetch_time_blocked);
-#endif
 }
 
 int
@@ -332,13 +334,13 @@ fetch_account(struct account *a, struct io *pio, int nflags, double tim)
 	struct msgbuf	 msgbuf;
 	struct fetch_ctx fctx;
 	struct cache	*cache;
-	struct iolist 	 iol;
+	struct iolist	 iol;
 	int		 aborted, complete, holding, timeout;
 
 	log_debug2("%s: fetching", a->name);
 
 	TAILQ_INIT(&fetch_matchq);
- 	TAILQ_INIT(&fetch_deliverq);
+	TAILQ_INIT(&fetch_deliverq);
 	fetch_queued = fetch_dropped = fetch_kept = 0;
 
 	if (nflags & FETCH_POLL && a->fetch->total == NULL) {
@@ -357,6 +359,8 @@ fetch_account(struct account *a, struct io *pio, int nflags, double tim)
 
 	aborted = complete = holding = 0;
 	for (;;) {
+		log_debug3("%s: fetch loop start", a->name);
+
 		if (sigusr1) {
 			log_debug("%s: caught SIGUSR1", a->name);
 			if (!(nflags & FETCH_POLL))
@@ -403,26 +407,35 @@ fetch_account(struct account *a, struct io *pio, int nflags, double tim)
 				fctx.flags |= FETCH_EMPTY;
 
 			/* Call the fetch function. */
+			log_debug3("%s: calling fetch state (%p, flags 0x%02x)",
+			    a->name, fctx.state, fctx.flags);
 			switch (fctx.state(a, &fctx)) {
 			case FETCH_ERROR:
 				/* Fetch error. */
+				log_debug3("%s: fetch, error", a->name);
 				goto abort;
 			case FETCH_EXIT:
 				/* Fetch completed. */
+				log_debug3("%s: fetch, exit", a->name);
 				complete = 1;
 				break;
 			case FETCH_AGAIN:
 				/* Fetch again - no blocking. */
+				log_debug3("%s: fetch, again", a->name);
 				continue;
 			case FETCH_BLOCK:
 				/* Fetch again - allow blocking. */
+				log_debug3("%s: fetch, block", a->name);
 				break;
 			case FETCH_MAIL:
 				/* Mail ready. */
+				log_debug3("%s: fetch, mail", a->name);
 				if (fetch_enqueue(a, pio, fctx.mail) != 0)
 					goto abort;
 				fctx.mail = xcalloc(1, sizeof *fctx.mail);
 				continue;
+			default:
+				fatalx("unexpected fetch return");
 			}
 		}
 
@@ -499,11 +512,11 @@ fetch_enqueue(struct account *a, struct io *pio, struct mail *m)
 {
 	struct mail_ctx		*mctx;
 	char			*hdr, rtime[128], *rhost, total[16];
-	u_int		 	 n, b;
-	size_t		 	 size;
-	int		 	 error;
- 	struct tm		*tm;
-	time_t		 	 t;
+	u_int			 n, b;
+	size_t			 size;
+	int			 error;
+	struct tm		*tm;
+	time_t			 t;
 
 	/*
 	 * Check for oversize mails. This must be first since there is no
@@ -525,13 +538,13 @@ fetch_enqueue(struct account *a, struct io *pio, struct mail *m)
 		return (0);
 	}
 
- 	/*
+	/*
 	 * Find the mail body (needed by trim_from). This is probably slower
 	 * than doing it during fetching but it guarantees consistency.
 	 */
 	m->body = find_body(m);
 
- 	/* Trim "From" line, if any. */
+	/* Trim "From" line, if any. */
 	trim_from(m);
 
 	/* Check for empty mails. */
@@ -546,7 +559,7 @@ fetch_enqueue(struct account *a, struct io *pio, struct mail *m)
 	m->tim = get_time();
 
 	/* Add account name tag. */
- 	add_tag(&m->tags, "account", "%s", a->name);
+	add_tag(&m->tags, "account", "%s", a->name);
 
 	/* Add mail time tags. */
 	if (mailtime(m, &t) != 0) {
@@ -564,7 +577,7 @@ fetch_enqueue(struct account *a, struct io *pio, struct mail *m)
 		add_tag(&m->tags, "mail_dayofweek", "%d", tm->tm_wday);
 		add_tag(&m->tags, "mail_dayofyear", "%.2d", tm->tm_yday + 1);
 		add_tag(&m->tags,
-		    "mail_quarter", "%d", (tm->tm_mon - 1) / 3 + 1);
+		    "mail_quarter", "%d", tm->tm_mon / 3 + 1);
 	}
 	if (rfc822time(t, rtime, sizeof rtime) != NULL)
 		add_tag(&m->tags, "mail_rfc822date", "%s", rtime);
@@ -602,7 +615,7 @@ fetch_enqueue(struct account *a, struct io *pio, struct mail *m)
 				rhost = conf.host_name;
 
 			error = insert_header(m, "received", "Received: by "
-			    "%.450s (%s " BUILD ", account \"%.450s\");\n\t%s",
+			    "%.450s (%s " VERSION ", account \"%.450s\");\n\t%s",
 			    rhost, __progname, a->name, rtime);
 		}
 		if (error != 0)
