@@ -1,4 +1,4 @@
-/* $Id: fdm.c,v 1.185 2011/09/14 13:36:19 nicm Exp $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -268,17 +268,18 @@ usage(void)
 	fprintf(stderr,
 	    "usage: %s [-hklmnqv] [-a name] [-D name=value] [-f conffile] "
 	    "[-u user] [-x name] [fetch|poll|cache] [arguments]\n", __progname);
-        exit(1);
+	exit(1);
 }
 
 int
 main(int argc, char **argv)
 {
-        int		 opt, lockfd, status, res;
+	int		 opt, lockfd, status, res;
 	u_int		 i;
-	enum fdmop       op = FDMOP_NONE;
+	enum fdmop	 op = FDMOP_NONE;
 	const char	*proxy = NULL, *s;
 	char		 tmp[BUFSIZ], *ptr, *lock = NULL, *user, *home = NULL;
+	long long	 used = 0;
 	struct utsname	 un;
 	struct passwd	*pw;
 	struct stat	 sb;
@@ -288,7 +289,7 @@ main(int argc, char **argv)
 	pid_t		 pid;
 	struct children	 children, dead_children;
 	struct child	*child;
-	struct io       *dead_io;
+	struct io	*dead_io;
 	struct iolist	 iol;
 	double		 tim;
 	struct sigaction act;
@@ -311,6 +312,8 @@ main(int argc, char **argv)
 	TAILQ_INIT(&conf.rules);
 	TAILQ_INIT(&conf.actions);
 	TAILQ_INIT(&conf.caches);
+	conf.lock_wait = 0;
+	conf.lock_timeout = DEFLOCKTIMEOUT;
 	conf.max_size = DEFMAILSIZE;
 	conf.timeout = DEFTIMEOUT;
 	conf.lock_types = LOCK_FLOCK;
@@ -333,18 +336,18 @@ main(int argc, char **argv)
 	ARRAY_INIT(&conf.excl);
 
 	ARRAY_INIT(&macros);
-        while ((opt = getopt(argc, argv, "a:D:f:hklmnqu:vx:")) != -1) {
-                switch (opt) {
+	while ((opt = getopt(argc, argv, "a:D:f:hklmnqu:vx:")) != -1) {
+		switch (opt) {
 		case 'a':
 			ARRAY_ADD(&conf.incl, xstrdup(optarg));
 			break;
 		case 'D':
 			ARRAY_ADD(&macros, optarg);
 			break;
-                case 'f':
+		case 'f':
 			if (conf.conf_file == NULL)
 				conf.conf_file = xstrdup(optarg);
-                        break;
+			break;
 		case 'h':
 			home = getenv("HOME");
 			break;
@@ -364,20 +367,20 @@ main(int argc, char **argv)
 			if (conf.def_user == NULL)
 				conf.def_user = xstrdup(optarg);
 			break;
-                case 'v':
+		case 'v':
 			if (conf.debug != -1)
 				conf.debug++;
-                        break;
+			break;
 		case 'q':
 			conf.debug = -1;
 			break;
 		case 'x':
 			ARRAY_ADD(&conf.excl, xstrdup(optarg));
 			break;
-                default:
-                        usage();
-                }
-        }
+		default:
+			usage();
+		}
+	}
 	argc -= optind;
 	argv += optind;
 	if (conf.check_only) {
@@ -406,7 +409,7 @@ main(int argc, char **argv)
 	else
 		log_open_tty(conf.debug);
 	tt = time(NULL);
-	log_debug("version is: %s " BUILD ", started at: %.24s", __progname,
+	log_debug("version is: %s " VERSION ", started at: %.24s", __progname,
 	    ctime(&tt));
 
 	/* And the OS version. */
@@ -426,7 +429,7 @@ main(int argc, char **argv)
 		log_warnx("unknown user: %lu", (u_long) geteuid());
 		exit(1);
 	}
- 	user = xstrdup(pw->pw_name);
+	user = xstrdup(pw->pw_name);
 	if (home != NULL && *home != '\0')
 		conf.user_home = xstrdup(home);
 	else
@@ -445,13 +448,13 @@ main(int argc, char **argv)
 	}
 	log_debug2("loading configuration from %s", conf.conf_file);
 	if (stat(conf.conf_file, &sb) == -1) {
-                log_warn("%s", conf.conf_file);
+		log_warn("%s", conf.conf_file);
 		exit(1);
 	}
 	if (geteuid() != 0 && (sb.st_mode & (S_IROTH|S_IWOTH)) != 0)
 		log_warnx("%s: world readable or writable", conf.conf_file);
-        if (parse_conf(conf.conf_file, &macros) != 0) {
-                log_warn("%s", conf.conf_file);
+	if (parse_conf(conf.conf_file, &macros) != 0) {
+		log_warn("%s", conf.conf_file);
 		exit(1);
 	}
 	ARRAY_FREE(&macros);
@@ -471,7 +474,7 @@ main(int argc, char **argv)
 		conf.queue_low = conf.queue_high * 3 / 4;
 		if (conf.queue_low >= conf.queue_high)
 			conf.queue_low = conf.queue_high - 1;
- 	}
+	}
 
 	/* Set the umask. */
 	umask(conf.file_umask);
@@ -538,6 +541,8 @@ main(int argc, char **argv)
 		off = strlcat(tmp, "delete-oversized, ", sizeof tmp);
 	if (conf.verify_certs)
 		off = strlcat(tmp, "verify-certificates, ", sizeof tmp);
+	if (conf.ignore_errors)
+		off = strlcat(tmp, "ignore-errors, ", sizeof tmp);
 	if (sizeof tmp > off && conf.purge_after > 0) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "purge-after=%u, ", conf.purge_after);
@@ -583,12 +588,18 @@ main(int argc, char **argv)
 	}
 	if (sizeof tmp > off && conf.max_accts != -1) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
-		    "parallel-accounts=%d ", conf.max_accts);
+		    "parallel-accounts=%d, ", conf.max_accts);
 	}
 	if (sizeof tmp > off && conf.lock_file != NULL) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "lock-file=\"%s\", ", conf.lock_file);
 	}
+	if (sizeof tmp > off && conf.lock_timeout != DEFLOCKTIMEOUT) {
+		off += xsnprintf(tmp + off, (sizeof tmp) - off,
+		    "lock-timeout=%d, ", conf.lock_timeout);
+	}
+	if (conf.lock_wait)
+		off = strlcat(tmp, "lock-wait, ", sizeof tmp);
 	if (sizeof tmp > off) {
 		off += xsnprintf(tmp + off, (sizeof tmp) - off,
 		    "strip-characters=\"%s\", ", conf.strip_chars);
@@ -619,12 +630,12 @@ main(int argc, char **argv)
 	/* If -n, bail now, otherwise check there is something to work with. */
 	if (conf.check_only)
 		exit(0);
-        if (TAILQ_EMPTY(&conf.accounts)) {
-                log_warnx("no accounts specified");
+	if (TAILQ_EMPTY(&conf.accounts)) {
+		log_warnx("no accounts specified");
 		exit(1);
 	}
-        if (op == FDMOP_FETCH && TAILQ_EMPTY(&conf.rules)) {
-                log_warnx("no rules specified");
+	if (op == FDMOP_FETCH && TAILQ_EMPTY(&conf.rules)) {
+		log_warnx("no rules specified");
 		exit(1);
 	}
 
@@ -687,9 +698,15 @@ main(int argc, char **argv)
 		else
 			xasprintf(&lock, "%s/%s", conf.user_home, LOCKFILE);
 	}
+retry:
 	if (*lock != '\0' && !conf.allow_many) {
 		lockfd = xcreate(lock, O_WRONLY, -1, -1, S_IRUSR|S_IWUSR);
 		if (lockfd == -1 && errno == EEXIST) {
+			if (conf.lock_wait) {
+				if (locksleep(NULL, lock, &used) == 0)
+					goto retry;
+				exit(1);
+			}
 			log_warnx("already running (%s exists)", lock);
 			exit(1);
 		} else if (lockfd == -1) {
@@ -700,12 +717,8 @@ main(int argc, char **argv)
 	}
 	conf.lock_file = lock;
 
-        SSL_library_init();
-        SSL_load_error_strings();
-
-#ifdef DEBUG
-	COUNTFDS("parent");
-#endif
+	SSL_library_init();
+	SSL_load_error_strings();
 
 	/* Filter account list. */
 	TAILQ_INIT(&actaq);
@@ -714,7 +727,7 @@ main(int argc, char **argv)
 			TAILQ_INSERT_HEAD(&actaq, a, active_entry);
 	}
 	if (TAILQ_EMPTY(&actaq)) {
-                log_warnx("no accounts found");
+		log_warnx("no accounts found");
 		res = 1;
 		goto out;
 	}
@@ -815,9 +828,11 @@ main(int argc, char **argv)
 				    (long) child->pid);
 				kill(child->pid, SIGTERM);
 
-				io_close(child->io);
-				io_free(child->io);
-				child->io = NULL;
+				if (child->io != NULL) {
+					io_close(child->io);
+					io_free(child->io);
+					child->io = NULL;
+				}
 			}
 		}
 	}
@@ -849,8 +864,10 @@ main(int argc, char **argv)
 			child = ARRAY_ITEM(&children, i);
 			kill(child->pid, SIGTERM);
 
-			io_close(child->io);
-			io_free(child->io);
+			if (child->io != NULL) {
+				io_close(child->io);
+				io_free(child->io);
+			}
 			xfree(child);
 		}
 		ARRAY_FREE(&children);
@@ -869,25 +886,23 @@ main(int argc, char **argv)
 	}
 
 	tim = get_time() - tim;
- 	log_debug2("parent: finished, total time %.3f seconds", tim);
+	log_debug2("parent: finished, total time %.3f seconds", tim);
 
 out:
 	if (!conf.allow_many && *conf.lock_file != '\0')
 		unlink(conf.lock_file);
 
 #ifdef DEBUG
-	COUNTFDS("parent");
-
 	/* Free everything. */
 	if (conf.proxy != NULL) {
 		if (conf.proxy->user != NULL)
 			xfree(conf.proxy->user);
 		if (conf.proxy->pass != NULL)
- 			xfree(conf.proxy->pass);
+			xfree(conf.proxy->pass);
 		if (conf.proxy->server.host != NULL)
- 			xfree(conf.proxy->server.host);
+			xfree(conf.proxy->server.host);
 		if (conf.proxy->server.port != NULL)
- 			xfree(conf.proxy->server.port);
+			xfree(conf.proxy->server.port);
 		xfree(conf.proxy);
 	}
 	while (!TAILQ_EMPTY(&conf.caches)) {
@@ -925,8 +940,6 @@ out:
 	xfree(conf.strip_chars);
 	free_strings(&conf.incl);
 	free_strings(&conf.excl);
-
-	xmalloc_report(getpid(), "parent");
 #endif
 
 	exit(res);
