@@ -1,4 +1,4 @@
-/* $Id: fetch-mbox.c,v 1.12 2007/12/08 20:22:36 nicm Exp $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -302,7 +302,7 @@ fetch_mbox_state_open(struct account *a, struct fetch_ctx *fctx)
 	struct fetch_mbox_data	*data = a->data;
 	struct fetch_mbox_mbox	*fmbox;
 	char			*ptr;
-	struct stat	         sb;
+	struct stat		 sb;
 	uintmax_t		 size;
 	long long		 used;
 
@@ -434,6 +434,9 @@ fetch_mbox_state_mail(struct account *a, struct fetch_ctx *fctx)
 
 	/* Tag mail. */
 	default_tags(&m->tags, NULL);
+	add_tag(&m->tags, "mbox", "%s", xbasename(fmbox->path));
+	add_tag(&m->tags, "mbox_path", "%s", xdirname(fmbox->path));
+	add_tag(&m->tags, "mbox_file", "%s", fmbox->path);
 
 	/*
 	 * We start at a "From " line and include it in the mail (it can be
