@@ -1,4 +1,4 @@
-/* $Id: match-unmatched.c,v 1.4 2006/11/28 18:14:33 nicm Exp $ */
+/* $Id$ */
 
 /*
  * Copyright (c) 2006 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -21,22 +21,27 @@
 #include <string.h>
 
 #include "fdm.h"
+#include "match.h"
 
-int	unmatched_match(struct match_ctx *, struct expritem *);
-char   *unmatched_desc(struct expritem *);
+int	match_unmatched_match(struct mail_ctx *, struct expritem *);
+void	match_unmatched_desc(struct expritem *, char *, size_t);
 
-struct match match_unmatched = { unmatched_match, unmatched_desc };
+struct match match_unmatched = {
+	"unmatched",
+	match_unmatched_match,
+	match_unmatched_desc
+};
 
 int
-unmatched_match(struct match_ctx *mctx, unused struct expritem *ei)
+match_unmatched_match(struct mail_ctx *mctx, unused struct expritem *ei)
 {
 	if (mctx->matched)
 		return (MATCH_FALSE);
 	return (MATCH_TRUE);
 }
 
-char *
-unmatched_desc(unused struct expritem *ei)
+void
+match_unmatched_desc(unused struct expritem *ei, char *buf, size_t len)
 {
-	return (xstrdup("unmatched"));
+	strlcpy(buf, "unmatched", len);
 }
